# -*- coding: utf-8 -*-
"""
@author: Daniel Espinosa- Daespinosag

"""
import tensorflow as tf
import tempfile

class KerasCacheable():
    def get_params_cacheable(self):
        '''Esta funcion se usa para extraer los parametros cachables y configurar el guardado y la extracción.
            -> este metodo debe sobreescribirse si no se quieren tomar todas las propiedades como cachables
            -> Si sobre escribe este metodo asegurese de escribir correctamente los nombres de sus propiedades si ingresa algun nombre errado el proceso de ejecusión fallará
                return:
                    properties: Se retorna una lista con los nombres de las propiedades que son cacheables'''    
        
        return self.get_params().keys()

    def get_custom_objects():
        '''
        Esta funcion se usa para extraer las acciones custom que debe ejecutar el modelo.
            -> este metodo debe sobreescribirse se se quiere incluir una funcion custom para ejecutar el modelo (por ejemplo: una funcion de costo)
                return:
                    properties: Un diccionario con: {'nombre_de_la_funcion' : referencia a la funcion }
        '''
        return {}

    def __getstate__(self):
        '''
        Este metodo mágico se utiliza para extraer la información cacheada 
            return:
                es un diccionario {'propiedad' : valor_de_la_propiedad }    
        '''
        model_str = ""
        
        with tempfile.NamedTemporaryFile(suffix='.h5', delete=True) as fd:
            try:
                self.model.save(fd.name,save_format='h5')
                model_str = fd.read()
            except:
                model_str  = 'model no trained'
                self.A_out = 'model no trained'
                self.A_in  = 'model no trained'

        d = { 'model_str': model_str,'A_in': self.A_in,'A_out': self.A_out }

        for att in self.get_params_cacheable():
            d[att] = getattr(self,att)            
        
        return d

    def __setstate__(self, state):
        '''
        Este metodo mágico se utiliza para actualizar la información cacheada 
        '''
        for att in self.get_params_cacheable():
            setattr(self,att,state[att])

        with tempfile.NamedTemporaryFile(suffix='.h5', delete=True) as fd:
            try:
                fd.write(state['model_str'])
                fd.flush()
                self.model=tf.keras.models.load_model(fd.name,custom_objects=self.get_custom_objects())    
                self.A_in=state['A_in']
                self.A_out=state['A_out']
            except:
                self.mode=state['model_str']
         
                




import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="keras-grid-search-cacheable",
    version="1.0.0",
    author="Daniel Espinosa",
    author_email="daespinosag@unal.edu.co",
    description="Reducción de tiempo de ejecución de los algoritmos de Machine Learning con búsqueda de parámetros en GridSearch.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/machine-learning-tools/keras-grid-search-cacheable",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6.9',
    install_requires=[
        'tensorflow>=1.15.0'
    ]
)
